% calculate the length of one ray in given block
% In:
%   top_left is the top left position of block, e.g. [0 0] for the first
%   block
%
%   station is the location of station, e.g. 1 for station located at [1 0]
%
%   angle is angle of given ray respect to negtive oretation of x-axis

function length=getlength(top_left,station,angle)
    % x*sin + y*cos = x0*sin
    sina = sind(angle);
    cosa = cosd(angle);
    if (abs(cosa) < 0.01 || abs(sina) < 0.01)
        length = -1.0;
        return;
    end
    x_top   = (station*sina - top_left(2)*cosa)/sina;
    x_bot   = (station*sina - (top_left(2)+1)*cosa)/sina;
    y_left  = (station*sina - top_left(1)*sina)/cosa;
    y_right = (station*sina - (top_left(1)+1)*sina)/cosa;

    if (x_top > top_left(1) && x_top < (top_left(1)+1) )
        if (x_bot > top_left(1) && x_bot < (top_left(1)+1) )
            length = 1.0/sina;
            return;
        elseif (x_bot <= top_left(1))
            offset = x_top - top_left(1);
            length = offset/cosa;
            return;
        else
            offset = top_left(1)+1-x_top;
            length = -1.0*offset/cosa;
            return;
        end
    elseif (x_bot > top_left(1) && x_bot < (top_left(1)+1))
        if (x_top <= top_left(1))
            offset = x_bot - top_left(1);
            length = -1.0*offset/cosa;
            return
        else 
            offset = top_left(1)+1-x_bot;
            length = offset/cosa;
            return;
        end
    elseif ( (y_left >= top_left(2) && y_left <= (top_left(2)+1)) )
        if ( (y_right >= top_left(2) && y_right <= (top_left(2)+1) ) )
            length = 1.0/abs(cosa);
            return;
        end 
    end
    length = 0.0;
end